import socket

import httpx


def get_current_ip_address() -> str:
    host = "ifconfig.co"
    addrs = socket.getaddrinfo(host, 80, family=socket.AF_INET, proto=socket.IPPROTO_TCP)
    for addr in addrs:
        ipv4_addr = addr[4][0]
        resp = httpx.get("http://" + ipv4_addr + "/ip", headers={"Host": host})
        resp.raise_for_status()
        return resp.text.strip()
    raise Exception("Failed to get the current IP address")



import datetime


def timestamp_to_string(timestamp: int) -> str:
    """
    Convert timestamp to datetime string.
    Args:
        timestamp:

    Returns:
        the datetime string.

    """
    if timestamp is None:
        return ""
    return datetime.datetime.fromtimestamp(timestamp).isoformat()


def get_current_year_month() -> str:
    """
    Get current year and month.
    Returns:
        the year and month string in format of YYYY-MM.

    """
    return datetime.datetime.now().strftime("%Y-%m")



from .backup import Backup
from .cluster import Cluster
from .context import Context
from .exception import TiDBCloudException
from .project import Project
from .restore import Restore
from .tidbcloud import TiDBCloud
from .util.log import log



import httpx

from tidbcloudy.exception import TiDBCloudResponseException


class Context:

    def __init__(self, public_key: str, private_key: str, server_config: dict):
        """
        Args:
            public_key: your public key to access to TiDB Cloud
            private_key: your private key to access to TiDB Cloud
            server_config: the server configuration dict to access to TiDB Cloud
        """
        self._client = httpx.Client()
        self._client.auth = httpx.DigestAuth(public_key, private_key)
        self._server_config = server_config

    def _call_api(self, method: str, path: str, server: str, **kwargs) -> dict:
        base_url = self._server_config.get(server)
        if base_url[-1] != "/":
            base_url += "/"
        try:
            resp = self._client.request(method=method, url=base_url + path, **kwargs)
            resp.raise_for_status()
            return resp.json()
        except httpx.RequestError as exc:
            raise TiDBCloudResponseException(status="Error",
                                             message=f"An error occurred when requesting {exc.request.url}")
        except httpx.HTTPStatusError as exc:
            raise TiDBCloudResponseException(status=exc.response.status_code, message=exc.response.text)

    def call_get(self, server: str, path: str,
                 *,
                 params: dict = None) -> dict:
        resp = self._call_api(method="GET", path=path, server=server, params=params)
        return resp

    def call_post(self, server: str, path: str,
                  *,
                  data: dict = None,
                  json: dict = None) -> dict:
        resp = self._call_api(method="POST", path=path, server=server, data=data, json=json)
        return resp

    def call_patch(self, server: str, path: str,
                   *,
                   data: dict = None,
                   json: dict = None) -> dict:
        resp = self._call_api(method="PATCH", path=path, server=server, data=data, json=json)
        return resp

    def call_delete(self, server: str, path: str) -> dict:
        resp = self._call_api(method="DELETE", server=server, path=path)
        return resp



from typing import Iterator, List

from tidbcloudy.context import Context
from tidbcloudy.project import Project
from tidbcloudy.specification import BillingMonthSummary, CloudSpecification
from tidbcloudy.util.page import Page
from tidbcloudy.util.timestamp import get_current_year_month

SERVER_CONFIG_DEFAULT = {
    "v1beta": "https://api.tidbcloud.com/api/v1beta/",
    "billing": "https://billing.tidbapi.com/v1beta1/"
}


class TiDBCloud:
    def __init__(self, public_key: str, private_key: str, server_config: dict = None):
        if server_config is None:
            server_config = SERVER_CONFIG_DEFAULT
        self._context = Context(public_key, private_key, server_config)

    def create_project(self, name: str, aws_cmek_enabled: bool = False, update_from_server: bool = False) -> Project:
        """
        Create a project.
        Args:
            name: the project name.
            aws_cmek_enabled: whether to enable AWS Customer-Managed Encryption Keys.
            update_from_server: whether to update the project info after creating.

        Returns:
            If the update_from_server is False, return a Project object with only the context and project_id.
            If the update_from_server is True, return a Project object with all the info.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                project = api.create_project(name="your_project_name", aws_cmek_enabled=False, update_from_server=True)
                print(project)
        """
        config = {
            "name": name,
            "aws_cmek_enabled": aws_cmek_enabled
        }
        resp = self._context.call_post(server="v1beta", path="projects", json=config)
        project_id = resp["id"]
        if update_from_server:
            return self.get_project(project_id=project_id, update_from_server=True)
        return Project(context=self._context, id=project_id)

    def get_project(self, project_id: str, update_from_server: bool = False) -> Project:
        """
        Get the project object by project_id.
        Args:
            project_id: the project id.
            update_from_server: whether to update the project info.

        Returns:
            If the update_from_server is False, return a Project object with only the context and project_id.
            If the update_from_server is True, return a Project object with all the info.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                project = api.get_project("your_project_id", update_from_server=True)
        """
        project = Project(context=self._context, id=project_id)
        if update_from_server:
            for item in self.iter_projects():
                if item.id == project_id:
                    return item
            raise ValueError("Project {} not found".format(project_id))
        return project

    def list_projects(self, page: int = None, page_size: int = None) -> Page[Project]:
        """
        List all projects.
        Args:
            page: the page number.
            page_size: the page size of each page.

        Returns:
            the projects list.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                projects = api.list_projects()
                total = projects.total
                for project in projects.items:
                    print(project) # This is a Project object

        """
        query = {}
        if page is not None:
            query["page"] = page
        if page_size is not None:
            query["page_size"] = page_size
        resp = self._context.call_get(server="v1beta", path="projects", params=query)
        return Page(
            [Project.from_object(self._context, item) for item in resp["items"]],
            page, page_size, resp["total"])

    def iter_projects(self, page_size: int = 10) -> Iterator[Project]:
        """
        Iterate all projects.
        Args:
            page_size: the page size of each page.

        Returns:
            the projects iterator.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                for project in api.iter_projects():
                    print(project) # This is a Project object

        """
        page = 1
        total = None
        while total is None or (page - 1) * page_size < total:
            projects = self.list_projects(page=page, page_size=page_size)
            total = projects.total
            for project in projects.items:
                yield project
            page += 1

    def list_provider_regions(self) -> List[CloudSpecification]:
        """
        List all provider regions.
        Returns:
            the provider regions list.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                regions = api.list_provider_regions()
                for spec in api.list_provider_regions():
                    print(spec) # This is a CloudSpecification object

        """
        resp = self._context.call_get(server="v1beta", path="clusters/provider/regions")
        return [CloudSpecification.from_object(obj=item) for item in resp["items"]]

    def get_monthly_bill(self, month: str) -> BillingMonthSummary:
        """
        Get the monthly billing.
        Args:
            month: the month of the bill, format: YYYY-MM or YYYYMM.
        Returns:
            the monthly billing.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                billing = api.get_monthly_bill(month="2023-08")
                print(billing)

        """
        if "-" not in month and len(month) == 6:
            month = f"{month[:4]}-{month[4:]}"
        path = f"bills/{month}"
        resp = self._context.call_get(server="billing", path=path)
        return BillingMonthSummary.from_object(self._context, resp)

    def get_current_month_bill(self) -> BillingMonthSummary:
        """
        Get the billing of current month.
        Returns:
            the current month billing.

        Examples:
            .. code-block:: python
                import tidbcloudy
                api = tidbcloudy.TiDBCloud(public_key="your_public_key", private_key="your_private_key")
                billing = api.get_current_month_bill()
                print(billing)

        """
        return self.get_monthly_bill(month=get_current_year_month())
